//
// Basic surface functionality..
//
//! \file tonc_surface.c
//! \author J Vijn
//! \date 20080409 - 20080409
//
/* === NOTES ===
*/

#include <string.h>
#include "tonc_surface.h"
#include "tonc_video.h"

// --------------------------------------------------------------------
// FUNCTIONS 
// --------------------------------------------------------------------


void srf_init(TSurface *srf, enum ESurfaceType type, const void *data, 
	uint width, uint height, uint bpp, u16 *pal)
{
	memset(srf, 0, sizeof(TSurface));
	srf->data= (u8*)data;
	srf->type= type;

	switch(type &~ SRF_ALLOCATED)
	{
	case SRF_CHR4:
		bpp= 4;
		width=  (width+7)&~7;
		height= (height+7)&~7;
		srf->pitch= width*bpp;
		break;

	case SRF_CHR8:
		bpp= 8;
		width=  (width+7)&~7;
		height= (height+7)&~7;
		srf->pitch= width*bpp;
		break;

	case SRF_BMP8:
		bpp= 8;
		srf->pitch= width;
		break;

	case SRF_BMP16:
		bpp= 16;
		srf->pitch= width*2;
		pal= NULL;
		break;			
	}

	srf->width= width;
	srf->height= height;

	if(pal != NULL)
	{
		srf->palSize= 1<<bpp;
		srf->palData= pal;
	}
}

//! Get the byte address of coordinates (\a x, \a y) on the surface.
void *srf_get_ptr(TSurface *srf, uint x, uint y)
{
	uint bpp= srf->bpp;
	switch(srf->type &~ SRF_ALLOCATED)
	{
	case SRF_CHR4:
	case SRF_CHR8:
		return &srf->data[(y>>3)*srf->pitch + (y&7)*bpp + (x&~7)*bpp + (x&7)*bpp/8];

	default:
		return &srf->data[y*srf->pitch + (x*bpp>>3)];
	}
}


// EOF
