//
// Tile renderer, var width/height, 1->4bpp tiles,
// recolored with transparency
//
//! \file chr4_drawg_b1cts.c
//! \author J Vijn
//! \date 20070621 - 20070725
//
/* === NOTES ===
  * 20070725: Skipping rendering if raw == 0 helps. A lot. Also, there 
    is more than one way to bitunpack and split between tiles. Which 
	method is faster is very platform dependent.
  * 20070723: Prepping dst stuff inside the drawg and passing along to 
	renc does NOT help (prolly due to Thumb). Try combining in 
	asm manually.
*/

#include "tonc_memdef.h"
#include "tonc_tte.h"


// --------------------------------------------------------------------
// FUNCTIONS
// --------------------------------------------------------------------

//! Render 1bpp fonts to 4bpp tiles
void chr4_drawg_b1cts(uint gid)
{
	TTE_BASE_VARS(tc, font);
	TTE_CHAR_VARS(font, gid, u8, srcD, srcL, charW, charH);
	uint x= tc->cursorX, y= tc->cursorY, dstP= tc->dst.pitch;
	uint srcP= font->cellH;

	u32 *dstD= (u32*)(tc->dst.data + y/8*dstP + (y%8)*4 + x/8*32), *dstL;
	x &= 7;
	dstP= dstP/4 - 8;
	u32 lsl= 4*x, lsr= 32-4*x, right= x+charW;

	// Inner loop vars
	u32 px, pxmask, raw;
	u32 ink= tc->cattr[TTE_INK];
	const u32 mask= 0x01010101;

	uint iy, iw;
	for(iw=0; iw<charW; iw += 8)	// Loop over strips
	{
		dstL= dstD;		dstD += 8;
		srcL= srcD;		srcD += srcP;

		iy= charH;
		while(iy--)					// Loop over scanlines
		{
			raw= *srcL++;
			if(raw)
			{
				raw |= raw<<12;
				raw |= raw<< 6;
				px   = raw & mask<<1;
				raw &= mask;
				px   = raw | px<<3;

				pxmask= px*15;
				px   *= ink;

				// Write left tile:
				dstL[0] = (dstL[0] &~ (pxmask<<lsl) ) | (px<<lsl);

				// Write right tile (if any)
				if(right > 8)
					dstL[8]= (dstL[8] &~ (pxmask>>lsr) ) | (px>>lsr);
			}
			dstL++;

			if( ((u32)dstL)%32 == 0 )
				dstL += dstP;
		}
	}
}



/* 20070907, obsolete. *slightly* faster, but otherwise outdated 
{
	// tc, font, gid, charW/H, cellH/S, srcD/L;
	TTE_BASE_VARS(tc, font);
	TTE_CHAR_VARS(font, gid, u8, srcD, srcL, charW, charH, cellH);

	while(charW>8)
	{
		chr4_renc_b1cts(srcL, 8, charH);
		tc->cursorX += 8;
		srcL += cellH;
		charW -= 8;
	}
	chr4_renc_b1cts(srcL, charW, charH);
	tc->cursorX += charW;

	return gid;
}

void chr4_renc_b1cts(const u8 *srcL, u32 charW, u32 charH)
{
	TTC *tc= tte_get_context();

	// Get destination parts
	u32 x= tc->cursorX, y= tc->cursorY, dstP= tc->dstPitch/4;
	u32 *dstL= tc->dstBase;
	dstL += y/8*dstP + y%8;
	dstL += x/8*8;
	x &= 7;
	dstP -= 8;

	// Loop parameters
	u32 ink= tc->colors[TTE_INK];
	u32 px, pxmask, raw;

	// Two versions: ARM and Thumb. Just for kicks (and speed)
#if __thumb__
	const u16 pxlut[16]= 
	{
		0x0000, 0x0001, 0x0010, 0x0011, 
		0x0100, 0x0101, 0x0110, 0x0111, 
		0x1000, 0x1001, 0x1010, 0x1011, 
		0x1100, 0x1101, 0x1110, 0x1111, 
	};

	// Fix colors
	while(charH--)
	{
		// Unpack pixels
		raw= *srcL++;
		if(raw)
		{
			raw <<= x;
			if( (px=raw&255) )
			{
				px    = pxlut[px&15] | pxlut[px>>4]<<16;
				pxmask= px*15;
				px   *= ink;
				dstL[0] = (dstL[0]&~pxmask) | px;
			}
			if( (px=raw>>8) )
			{
				px    = pxlut[px&15] | pxlut[px>>4]<<16;
				pxmask= px*15;
				px   *= ink;
				dstL[8] = (dstL[8]&~pxmask) | px;
			}
		}
		dstL++;

		if( ((u32)dstL)%32 == 0 )
			dstL += dstP;
	}

#else

	//# This *should* be the faster way in ARM, but only if done right.
	//# Haven't tested that part yet.

	u32 lsl= 4*x, lsr= 32-4*x, right= x+charW;
	const u32 mask= 0x01010101;

	// Fix colors
	while(charH--)
	{
		// Unpack pixels
		raw= *srcL++;
		if(raw)
		{
			raw |= raw<<12;
			raw |= raw<< 6;
			px   = raw & mask<<1;
			raw &= mask;
			raw |= px<<3;
			px		= raw*ink;
			pxmask	= raw*15;

			// Write right tile (if any)
			if(right > 8)
				dstL[8]= (dstL[8] &~ (pxmask>>lsr) ) | (px>>lsr);

			// Write left tile:
			dstL[0] = (dstL[0] &~ (pxmask<<lsl) ) | (px<<lsl);
		}
		dstL++;

		if( ((u32)dstL)%32 == 0 )
			dstL += dstP;
	}
#endif

}
*/


// EOF
