
//
// Initializer for 4bpp tiled text
//
//! \file tte_init_chr4.c
//! \author J Vijn
//! \date 20070517 - 20080315
//
/* === NOTES ===
  * 20080315,jv: separated clrids into se0 and cattrs fields.
	se0: screen-entry offset for map
	cattrs: packed BYTES (not nybbles) for color attributes.
*/

#include <string.h>
#include <tonc.h>
#include "tonc_tte.h"


// --------------------------------------------------------------------
// GLOBALS
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// FUNCTIONS
// --------------------------------------------------------------------


//! Initialize text system for 4bpp tile fonts.
/*!	
	\param bgnr		Background number.
	\param bgcnt	Background control flags.
	\param se0		Base offset for screen-entries.
	\param cattrs	Color attributes; one byte per attr.
	\param clrs		ink(/shadow) colors.
	\param font		Font to initialize with.
	\param proc		Glyph renderer
*/
void tte_init_chr4(int bgnr, u16 bgcnt, u16 se0, u32 cattrs, u32 clrs, 
	const TFont *font, fnDrawg proc)
{
	if(font==NULL)	font= &vwf_default;
	if(proc==NULL)	proc= chr4_drawg_default;

	tte_init_base(font, proc, chr4_erase);
	REG_BGCNT[bgnr]= bgcnt;

	TTC *tc= tte_get_context();
	TSurface *srf= &tc->dst;

	srf->data= (u8*)&tile_mem[BFN_GET(bgcnt, BG_CBB)][se0 & SE_ID_MASK];
	srf->pitch= SCREEN_WIDTH*8/2;
	srf->width= SCREEN_WIDTH;
	srf->height= SCREEN_HEIGHT;
	srf->bpp= 4;
	srf->type= SRF_CHR4;
	srf->palSize= 16;
	// srf->palData= pal_bg_mem;	// later

	tc->flags0= bgnr;
	tc->ctrl= bgcnt;

	// --- Init cattr ---
	u32 ink, shadow, paper, spec;
	ink=	 cattrs     & 15;
	shadow=	(cattrs>> 8)& 15;
	paper=	(cattrs>>16)& 15;
	spec=   (cattrs>>24)&255;

	tc->cattr[TTE_INK]= ink;
	tc->cattr[TTE_SHADOW]= shadow;
	tc->cattr[TTE_PAPER]= paper;
	//tc->cattr[TTE_SPECIAL]= 0;

	srf->palData= pal_bg_bank[se0>>12];
	srf->palData[ink]= clrs&0xFFFF;
	srf->palData[shadow]= clrs>>16;

	// --- Fill out screenblock ---
	//# PONDER: do elsewhere?

	schr4_prep_map(srf, se_mem[BFN_GET(bgcnt, BG_SBB)], se0);
}


//! Erase part of the 4bpp text canvas.
void chr4_erase(int left, int top, int right, int bottom)
{	
	TTC *tc= tte_get_context();

	//# Check for single sequence
	schr4_rect(&tc->dst, left, top, right, bottom, tc->cattr[TTE_PAPER]);
}

// EOF
